package com.ruoyi.project.training.newstaff.mapper;

import java.util.List;
import com.ruoyi.framework.aspectj.lang.annotation.DataScope;
import com.ruoyi.project.training.newstaff.domain.TTnNew;

/**
 * 新员工培训Mapper接口
 *
 * @author ruoyi
 * @date 2023-06-29
 */
public interface TTnNewMapper
{
    /**
     * 查询新员工培训
     *
     * @param newId 新员工培训ID
     * @return 新员工培训
     */
    public TTnNew selectTTnNewById(Long newId);

    /**
     * 查询新员工培训列表
     *
     * @param tTnNew 新员工培训
     * @return 新员工培训集合
     */
    @DataScope(deptAlias = "d")
    public List<TTnNew> selectTTnNewList(TTnNew tTnNew);

    /**
     * 新增新员工培训
     *
     * @param tTnNew 新员工培训
     * @return 结果
     */
    public int insertTTnNew(TTnNew tTnNew);

    /**
     * 修改新员工培训
     *
     * @param tTnNew 新员工培训
     * @return 结果
     */
    public int updateTTnNew(TTnNew tTnNew);

    /**
     * 删除新员工培训
     *
     * @param newId 新员工培训ID
     * @return 结果
     */
    public int deleteTTnNewById(Long newId);

    /**
     * 批量删除新员工培训
     *
     * @param newIds 需要删除的数据ID
     * @return 结果
     */
    public int deleteTTnNewByIds(Long[] newIds);
}
